/**********************************************************************************************/
/*                                                                                            */
/*                                    .                                         */
/*                                                                                            */
/**********************************************************************************************/
#include  "display.h"

/*--------------------------------------------------------------------------------------------*/
/*    .                                                        */
/*--------------------------------------------------------------------------------------------*/
//    1  ,     .
BYTE LCD_BackLight_Brightness = 15; //(MIN_BACKLIGHT_BRIGHTNESS + MAX_BACKLIGHT_BRIGHTNESS) / 2;
BYTE LCD_BackLight_ON_Cntr;
#define   LCD_BACKLIGHT_CYCLE   15

/*--------------------------------------------------------------------------------------------*/
/*     .                                           */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void LCD_BackLight_Timer_Proc(void)
{
  if(LCD_BackLight_ON_Cntr-- == 0)
    {
      if(is_LCD_BackLight_ON())
        {
          LCD_BackLight_OFF();
          LCD_BackLight_ON_Cntr = LCD_BACKLIGHT_CYCLE - LCD_BackLight_Brightness;
        }
      else
        {
          LCD_BackLight_ON();
          LCD_BackLight_ON_Cntr = LCD_BackLight_Brightness;
        }
    }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   .                                                           */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Display_Init(void)
{
  //       :
  //     ,  CS1  CS2 = "1".
  DDRA |= (1 << LCD_E) | (1 << LCD_RW) | (1 << LCD_A0) | (1 << LCD_CS2) | (1 << LCD_CS1) | (1 << LCD_RES) | (1 << LCD_BACKLIGHT);
  PORTA &= ~((1 << LCD_E) | (1 << LCD_RW) | (1 << LCD_A0) | (1 << LCD_CS2) | (1 << LCD_CS1) | (1 << LCD_RES) | (1 << LCD_BACKLIGHT));
  PORTA |= (1 << LCD_CS1) | (1 << LCD_CS2);
  //    .   ,  
  //      .
  Clr_Line_in_Port(LCD_RES_PIN);
  __delay_cycles(1000);
  Set_Line_in_Port(LCD_RES_PIN);
  __delay_cycles(1000);
  //    .
  LCD_Clear();
  Write_Cmd_Display_ON();
  //  ,   .
  LCD_BackLight_ON_Cntr = 0;
  Start_Timer(LCD_BACKLIGHT_TIMER, 1, &LCD_BackLight_Timer_Proc, TIMER_CYCLE);
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*     .                                                */
/* :    (0, 1).                                                            */
/* :   .                                                          */
/*--------------------------------------------------------------------------------------------*/
OUT BYTE Read_Status_from_LCD(IN IDX c)
{
  //          .
  LCD_Data_Bus_To_Input();
  PORTA |= (1 << LCD_RW); PORTA &= ~((1 << LCD_A0) | (1 << LCD_E));
  (c == 0) ? (Clr_Line_in_Port(LCD_CS1_PIN)) : (Clr_Line_in_Port(LCD_CS2_PIN));
  //         .
  Set_Line_in_Port(LCD_E_PIN);
  __delay_cycles(10);
  register BYTE state = Get_Data_from_LCD_Data_Bus();
  Clr_Line_in_Port(LCD_E_PIN);
  //    .
  PORTA |= (1 << LCD_CS1) | (1 << LCD_CS2);
  return state;
}

/*--------------------------------------------------------------------------------------------*/
/*     .                                             */
/* :    (0, 1).                                                            */
/* :  true,   , false .                                     */
/*--------------------------------------------------------------------------------------------*/
OUT bool Wait_LCD_Ready(IN IDX c)
{
  register BYTE i = 0;
  //          .
  LCD_Data_Bus_To_Input();
  PORTA |= (1 << LCD_RW); PORTA &= ~((1 << LCD_A0) | (1 << LCD_E));
  (c == 0) ? (Clr_Line_in_Port(LCD_CS1_PIN)) : (Clr_Line_in_Port(LCD_CS2_PIN));
  //         .
  for(i = 0; i < 10; i++)
     {
       __delay_cycles(10);
       Set_Line_in_Port(LCD_E_PIN);
       __delay_cycles(10);
       register BYTE state = Get_Data_from_LCD_Data_Bus();
       Clr_Line_in_Port(LCD_E_PIN);
       if((state & (1 << 7)) == 0) break;
     }
  //    .
  PORTA |= (1 << LCD_CS1) | (1 << LCD_CS2);
  return (i < 10) ? (true) : (false);
}

/*--------------------------------------------------------------------------------------------*/
/*      .                                                */
/* :    (0, 1),   .                                  */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Cmd_to_LCD(IN IDX c, IN BYTE cmd)
{
  //    .
  Wait_LCD_Ready(c);
  //          .
  LCD_Data_Bus_to_Output();
  PORTA &= ~((1 << LCD_A0) | (1 << LCD_RW) | (1 << LCD_E));
  (c == 0) ? (Clr_Line_in_Port(LCD_CS1_PIN)) : (Clr_Line_in_Port(LCD_CS2_PIN));
  //          .
  Put_Data_to_LCD_Data_Bus(cmd);
  Set_Line_in_Port(LCD_E_PIN);
  __delay_cycles(10);
  Clr_Line_in_Port(LCD_E_PIN);
  //    .
  PORTA |= (1 << LCD_CS1) | (1 << LCD_CS2);
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   1     .                                          */
/* :    (0, 1),   .                                  */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Data_to_LCD(IN IDX c, IN BYTE data)
{
  //    .
  Wait_LCD_Ready(c);
  //          .
  LCD_Data_Bus_to_Output();
  PORTA |= (1 << LCD_A0); PORTA &= ~((1 << LCD_RW) | (1 << LCD_E));
  (c == 0) ? (Clr_Line_in_Port(LCD_CS1_PIN)) : (Clr_Line_in_Port(LCD_CS2_PIN));
  //          .
  Put_Data_to_LCD_Data_Bus(data);
  Set_Line_in_Port(LCD_E_PIN);
  __delay_cycles(10);
  Clr_Line_in_Port(LCD_E_PIN);
  //    .
  PORTA |= (1 << LCD_CS1) | (1 << LCD_CS2);
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   n     .                                         */
/* :    (0, 1),    ,   .    */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Data_to_LCD(IN IDX c, IN BYTE __flash *pData, IN BYTE n)
{
  //    .
  Wait_LCD_Ready(c);
  //          .
  LCD_Data_Bus_to_Output();
  PORTA |= (1 << LCD_A0); PORTA &= ~((1 << LCD_RW) | (1 << LCD_E));
  (c == 0) ? (Clr_Line_in_Port(LCD_CS1_PIN)) : (Clr_Line_in_Port(LCD_CS2_PIN));
  //   c        .
  for(register BYTE i = 0; i < n; i++)
     {
       Put_Data_to_LCD_Data_Bus(*(pData + i));
       Set_Line_in_Port(LCD_E_PIN);
       __delay_cycles(10);
       Clr_Line_in_Port(LCD_E_PIN);
       __delay_cycles(100);
     }
  //    .
  PORTA |= (1 << LCD_CS1) | (1 << LCD_CS2);
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   n     .                                         */
/* :    (0, 1),    ,   .    */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Data_to_LCD(IN IDX c, IN BYTE *pData, IN BYTE n)
{
  //    .
  Wait_LCD_Ready(c);
  //          .
  LCD_Data_Bus_to_Output();
  PORTA |= (1 << LCD_A0); PORTA &= ~((1 << LCD_RW) | (1 << LCD_E));
  (c == 0) ? (Clr_Line_in_Port(LCD_CS1_PIN)) : (Clr_Line_in_Port(LCD_CS2_PIN));
  //   c        .
  for(register BYTE i = 0; i < n; i++)
     {
       Put_Data_to_LCD_Data_Bus(*(pData + i));
       Set_Line_in_Port(LCD_E_PIN);
       __delay_cycles(10);
       Clr_Line_in_Port(LCD_E_PIN);
       __delay_cycles(100);
     }
  //    .
  PORTA |= (1 << LCD_CS1) | (1 << LCD_CS2);
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   n      .                                    */
/* :    (0, 1),  ,   .                      */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Data_to_LCD(IN IDX c, IN BYTE data, IN BYTE n)
{
  //    .
  Wait_LCD_Ready(c);
  //          .
  LCD_Data_Bus_to_Output();
  PORTA |= (1 << LCD_A0); PORTA &= ~((1 << LCD_RW) | (1 << LCD_E));
  (c == 0) ? (Clr_Line_in_Port(LCD_CS1_PIN)) : (Clr_Line_in_Port(LCD_CS2_PIN));
  //   c         .
  for(register BYTE i = 0; i < n; i++)
     {
       Put_Data_to_LCD_Data_Bus(data);
       Set_Line_in_Port(LCD_E_PIN);
       __delay_cycles(10);
       Clr_Line_in_Port(LCD_E_PIN);
       __delay_cycles(100);
     }
  //    .
  PORTA |= (1 << LCD_CS1) | (1 << LCD_CS2);
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   n     .                                            */
/* :      (/),                                  */
/*                  ,   .             */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Data_to_LCD(IN BYTE P, IN BYTE C, IN BYTE *pData, IN BYTE n)
{
  register BYTE x = C, i = 0;
  while(x < (C + n))
    {
      if(x < 64)
        {
          Write_Cmd_Page_Addr_Set(0, P);
          Write_Cmd_Column_Addr_Set(0, x);
          i = 64 - x;
          Write_Data_to_LCD(0, pData, i);
        }
      else
        {
          Write_Cmd_Page_Addr_Set(1, P);
          Write_Cmd_Column_Addr_Set(1, x - 64);
          i = (C + n) - x;
          Write_Data_to_LCD(1, pData, i);
        }
      x += i;
      pData += i;
    }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   .                                                                     */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void LCD_Clear(void)
{
  for(register IDX page = 0; page < 8; page++)
     {
       //  .
       Write_Cmd_Page_Addr_Set(0, page); Write_Cmd_Page_Addr_Set(1, page);
       //    .
       Write_Cmd_Column_Addr_Set(0, 0); Write_Cmd_Column_Addr_Set(1, 0);
       Write_Data_to_LCD(0, (BYTE)0x00, 64); Write_Data_to_LCD(1, (BYTE)0x00, 64);
     }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   1     .                                        */
/* :    (0, 1).                                                            */
/* :   .                                                               */
/*--------------------------------------------------------------------------------------------*/
OUT BYTE Read_Data_from_LCD(IN IDX c)
{
  //    .
  Wait_LCD_Ready(c);
  //          .
  LCD_Data_Bus_To_Input();
  PORTA |= (1 << LCD_A0) | (1 << LCD_RW); PORTA &= ~(1 << LCD_E);
  (c == 0) ? (Clr_Line_in_Port(LCD_CS1_PIN)) : (Clr_Line_in_Port(LCD_CS2_PIN));
  //   - dummy.
  Set_Line_in_Port(LCD_E_PIN);
  __delay_cycles(10);
  Clr_Line_in_Port(LCD_E_PIN);
  __delay_cycles(100);
  //        .
  Set_Line_in_Port(LCD_E_PIN);
  __delay_cycles(10);
  register BYTE data = Get_Data_from_LCD_Data_Bus();
  Clr_Line_in_Port(LCD_E_PIN);
  //    .
  PORTA |= (1 << LCD_CS1) | (1 << LCD_CS2);
  return data;
}

/*--------------------------------------------------------------------------------------------*/
/*   n     .                                       */
/* :    (0, 1),     ,  . .    */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Read_Data_from_LCD(IN IDX c, OUT BYTE *pData, IN BYTE n)
{
  //    .
  Wait_LCD_Ready(c);
  //          .
  LCD_Data_Bus_To_Input();
  PORTA |= (1 << LCD_A0) | (1 << LCD_RW); PORTA &= ~(1 << LCD_E);
  (c == 0) ? (Clr_Line_in_Port(LCD_CS1_PIN)) : (Clr_Line_in_Port(LCD_CS2_PIN));
  //   - dummy.
  Set_Line_in_Port(LCD_E_PIN);
  __delay_cycles(10);
  Clr_Line_in_Port(LCD_E_PIN);
  //          .
  for(register BYTE i = 0; i < n; i++)
     {
       __delay_cycles(100);
       Set_Line_in_Port(LCD_E_PIN);
       __delay_cycles(10);
       *(pData + i) = Get_Data_from_LCD_Data_Bus();
       Clr_Line_in_Port(LCD_E_PIN);
     }
  //    .
  PORTA |= (1 << LCD_CS1) | (1 << LCD_CS2);
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*    n  .                                               */
/* :      (/),                                  */
/*                  ,   .                 */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Read_Data_from_LCD(IN BYTE P, IN BYTE C, OUT BYTE *pData, IN BYTE n)
{
  register BYTE c = C, i = 0;
  while(c < (C + n))
    {
      if(c < 64)
        {
          Write_Cmd_Page_Addr_Set(0, P);
          Write_Cmd_Column_Addr_Set(0, c);
          i = 64 - c;
          Read_Data_from_LCD(0, pData, i);
        }
      else
        {
          Write_Cmd_Page_Addr_Set(1, P);
          Write_Cmd_Column_Addr_Set(1, c - 64);
          i = (C + n) - c;
          Read_Data_from_LCD(1, pData, i);
        }
      c += i;
      pData += i;
    }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*       .                                  */
/* :      (),                                  */
/*                 ,                                                  */
/*                    flash,                                     */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Draw_Bitmap(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H, IN BYTE __flash *pData, IN BYTE flags)
{
  //     .
  BYTE buf[MAX_WIDTH];
  //   ,     .
  if((X >= MAX_WIDTH) || (Y >= MAX_HEIGHT)) return;
  //      .
  BYTE w = ((BYTE)(X + W) < MAX_WIDTH) ? (W) : (MAX_WIDTH - X);
  //       .
  BYTE h = H;   //       .
  while(h > 0)
    {
      //    ,    .
      //      .
      BYTE page = Y / 8;
      if(page >= 8) break;
      //     .
      Read_Data_from_LCD(page, X, &buf[0], w);
      //  ,      
      //        .
      BYTE mask = 0x00, n = 0;
      for(BYTE i = 0; i < 8; i++)
         {
           mask >>= 1;
           if(((BYTE)(((page * 8) + i)) >= Y) && ((BYTE)(((page * 8) + i)) < (BYTE)(Y + h)))
             {
               mask |= 0x80; n++;
             }
         }
      mask = ~mask;
      //         
      //       .
      if(((H - h) % 8) != 0) n = min(8 - ((H - h) % 8), n);
      //      .
      BYTE __flash *p = pData + W * ((H - h) / 8);
      for(BYTE i = 0; i < w; i++)
         {
           BYTE data = *(p + i);
           //    - .
           if(((flags & DRAW_UNDERLINED) != 0) && ((h - n) <= 0)) data |= (1 << (n - 1));
           //    - .
           if((flags & DRAW_INVERTED) != 0) data = ~data;
           //      .
           if((Y % 8) == 0) data >>= ((H - h) % 8);
           else             data <<= (Y % 8);
           //      .
           data &= ~mask;
           if((flags & DRAW_XOR) != 0) buf[i] ^= data;
           else                        buf[i] = (buf[i] & mask) | data;
         }
      //       .
      Write_Data_to_LCD(page, X, &buf[0], w);
      //     .
      h -= n; Y += n;
    }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*     .                                       */
/* :         ,                 */
/*                   ,                                 */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Fill_Rect(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H, IN BYTE flags)
{
  //     .
  BYTE buf[MAX_WIDTH];
  //   ,     .
  if((X >= MAX_WIDTH) || (Y >= MAX_HEIGHT)) return;
  //       .
  if((BYTE)(X + W) > MAX_WIDTH) W = MAX_WIDTH - X;
  //        .
  BYTE h = H;   //       .
  while(h > 0)
    {
      //    ,    .
      //      .
      BYTE page = Y / 8;
      if(page > 8) break;
      //     .
      Read_Data_from_LCD(page, X, &buf[0], W);
      //  ,      
      //        .
      BYTE mask = 0x00, n = 0;
      for(BYTE i = 0; i < 8; i++)
         {
           mask >>= 1;
           if(((BYTE)(((page * 8) + i)) >= Y) && ((BYTE)(((page * 8) + i)) < (BYTE)(Y + h)))
             {
               mask |= 0x80; n++;
             }
         }
      //      .
      for(BYTE i = 0; i < W; i++) buf[i] = ((flags & DRAW_INVERTED) != 0) ? (buf[i] & ~mask) : (buf[i] | mask);
      //       .
      Write_Data_to_LCD(page, X, &buf[0], W);
      //     .
      h -= n; Y += n;
    }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*     .                                     */
/* :         ,                 */
/*                   .                                 */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Invert_Rect(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H)
{
  //     .
  BYTE buf[MAX_WIDTH];
  //   ,     .
  if((X >= MAX_WIDTH) || (Y >= MAX_HEIGHT)) return;
  //       .
  if((BYTE)(X + W) > MAX_WIDTH) W = MAX_WIDTH - X;
  //        .
  BYTE h = H;   //       .
  while(h > 0)
    {
      //    ,    .
      //      .
      BYTE page = Y / 8;
      if(page > 8) break;
      //     .
      Read_Data_from_LCD(page, X, &buf[0], W);
      //  ,      
      //        .
      BYTE mask = 0x00, n = 0;
      for(BYTE i = 0; i < 8; i++)
         {
           mask >>= 1;
           if(((BYTE)(((page * 8) + i)) >= Y) && ((BYTE)(((page * 8) + i)) < (BYTE)(Y + h)))
             {
               mask |= 0x80; n++;
             }
         }
      //   .
      for(BYTE i = 0; i < W; i++) buf[i] = (buf[i] & ~mask) | (~buf[i] & mask);
      //       .
      Write_Data_to_LCD(page, X, &buf[0], W);
      //     .
      h -= n; Y += n;
    }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*      .                                            */
/* :      (),                                        */
/*                 .                                                  */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Put_Pixel(IN BYTE X, IN BYTE Y, IN BYTE flags)
{
  //   ,     .
  if((X >= MAX_WIDTH) || (Y >= MAX_HEIGHT)) return;
  //   ,     .
  register BYTE byte = 0;
  if(X < 64) { Write_Cmd_Page_Addr_Set(0, Y / 8); Write_Cmd_Column_Addr_Set(0, X);      byte = Read_Data_from_LCD(0); }
  else       { Write_Cmd_Page_Addr_Set(1, Y / 8); Write_Cmd_Column_Addr_Set(1, X - 64); byte = Read_Data_from_LCD(1); }
  //     .
  if((flags & DRAW_INVERTED) != 0) byte &= ~(1 << (Y % 8));
  else                             byte |=  (1 << (Y % 8));
  //     .
  if(X < 64) { Write_Cmd_Column_Addr_Set(0, X);      Write_Data_to_LCD(0, byte); }
  else       { Write_Cmd_Column_Addr_Set(1, X - 64); Write_Data_to_LCD(1, byte); }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*       ( ).                */
/* :        (),                               */
/*                 .                                                */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Draw_Line(IN BYTE X1, IN BYTE Y1, IN BYTE X2, IN BYTE Y2, IN BYTE flags)
{
  //         .
  signed char dX = X2 - X1, dY = Y2 - Y1;
  //    (   X,    Y).
  signed char inc_x, inc_y;
  if(dX > 0) inc_x = 1; else if (dX == 0) inc_x = 0; else inc_x = -1;
  if(dY > 0) inc_y = 1; else if (dY == 0) inc_y = 0; else inc_y = -1;
  //    .       .
  dX = abs(dX);
  dY = abs(dY);
  BYTE distance = (dX > dY) ? (dX) : (dY);
  //  .
  for(BYTE i = 0, x = 0, y = 0; i <= distance + 1; i++)
     {
       Put_Pixel(X1, Y1, flags);
       x += dX; y += dY;
       if(x > distance)
         {
           x -= distance;
           X1 += inc_x;
         }
       if(y > distance)
         {
           y -= distance;
           Y1 += inc_y;
         }
     }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*        .                          */
/* :      ,                                                           */
/*                 (),                                      */
/*                   flash,                                     */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Draw_Text(IN char *str, IN BYTE X, IN BYTE Y, IN struct FONT __flash *font, IN BYTE flags)
{
  //         (0x00).
  while(*str != 0)
    {
      //   .
      Draw_Char(*str, X, Y, font, flags);
      str++;
      X += font->Width;
    }
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*        .                          */
/* :      flash,                                                        */
/*                 (),                                      */
/*                   flash,                                     */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Draw_Text(IN char __flash *str, IN BYTE X, IN BYTE Y, IN struct FONT __flash *font, IN BYTE flags)
{
  //         (0x00).
  while(*str != 0)
    {
      //   .
      Draw_Char(*str, X, Y, font, flags);
      str++;
      X += font->Width;
    }
  return;
}

/**********************************************************************************************/
