/**********************************************************************************************/
/*                                                                                            */
/*                              .                                 */
/*                                                                                            */
/**********************************************************************************************/
#ifndef DISPLAY_H
#define DISPLAY_H

#include  "../main/main.h"
#include  "../timer/timer.h"          //    .

/*--------------------------------------------------------------------------------------------*/
/*  E  () (PA7).                                                                 */
/*--------------------------------------------------------------------------------------------*/
//   - .
#define   LCD_E               7
#define   LCD_E_PIN           A, LCD_E

/*--------------------------------------------------------------------------------------------*/
/*  A0  (/) (PA6).                                                       */
/*--------------------------------------------------------------------------------------------*/
// "1" -   , "0" -   .
#define   LCD_A0              6
#define   LCD_A0_PIN          A, LCD_A0

/*--------------------------------------------------------------------------------------------*/
/*  RW  (/) (PA5).                                                        */
/*--------------------------------------------------------------------------------------------*/
// "1" -  , "0" -  .
#define   LCD_RW              5
#define   LCD_RW_PIN          A, LCD_RW

/*--------------------------------------------------------------------------------------------*/
/*  CS2  ( 2) (PA4).                                                           */
/*--------------------------------------------------------------------------------------------*/
//   - .
#define   LCD_CS2             4
#define   LCD_CS2_PIN         A, LCD_CS2

/*--------------------------------------------------------------------------------------------*/
/*  CS1  ( 1) (PA3).                                                           */
/*--------------------------------------------------------------------------------------------*/
//   - .
#define   LCD_CS1             3
#define   LCD_CS1_PIN         A, LCD_CS1

/*--------------------------------------------------------------------------------------------*/
/*  RES  () (PA2).                                                               */
/*--------------------------------------------------------------------------------------------*/
//   - .
#define   LCD_RES             2
#define   LCD_RES_PIN         A, LCD_RES

/*--------------------------------------------------------------------------------------------*/
/*    (PA1).                                                         */
/*--------------------------------------------------------------------------------------------*/
// "1" -  , "0" -  .
#define   LCD_BACKLIGHT         1
#define   LCD_BACKLIGHT_PIN     A, LCD_BACKLIGHT
#define   LCD_BackLight_ON()    Set_Line_in_Port(LCD_BACKLIGHT_PIN)
#define   LCD_BackLight_OFF()   Clr_Line_in_Port(LCD_BACKLIGHT_PIN)
#define   is_LCD_BackLight_ON() is_Output_Line_in_Port_Set(LCD_BACKLIGHT_PIN)

/*--------------------------------------------------------------------------------------------*/
/*    (PC).                                                                      */
/*--------------------------------------------------------------------------------------------*/
#define   LCD_Data_Bus_to_Output()        DDRC = 0xff
#define   LCD_Data_Bus_To_Input()         { DDRC = 0x00; PORTC = 0xff; }
#define   Put_Data_to_LCD_Data_Bus(data)  PORTC = (data)
#define   Get_Data_from_LCD_Data_Bus()    PINC

/*--------------------------------------------------------------------------------------------*/
/*     .                                                           */
/*--------------------------------------------------------------------------------------------*/
#define   MAX_WIDTH     128
#define   MAX_HEIGHT    64

/*--------------------------------------------------------------------------------------------*/
/*    .                                                        */
/*--------------------------------------------------------------------------------------------*/
//    1  ,     .
#define   MAX_BACKLIGHT_BRIGHTNESS      15
#define   MIN_BACKLIGHT_BRIGHTNESS      0
extern BYTE LCD_BackLight_Brightness;

/*--------------------------------------------------------------------------------------------*/
/*     .                                           */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void LCD_BackLight_Timer_Proc(void);

/*--------------------------------------------------------------------------------------------*/
/*   .                                                           */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Display_Init(void);

/*--------------------------------------------------------------------------------------------*/
/*     .                                             */
/* :    (0, 1).                                                            */
/* :  true,   , false .                                     */
/*--------------------------------------------------------------------------------------------*/
OUT bool Wait_LCD_Ready(IN IDX c);

/*--------------------------------------------------------------------------------------------*/
/*     .                                                */
/* :    (0, 1).                                                            */
/* :   .                                                          */
/*--------------------------------------------------------------------------------------------*/
OUT BYTE Read_Status_from_LCD(IN IDX c);

/*--------------------------------------------------------------------------------------------*/
/*      .                                                */
/* :    (0, 1),   .                                  */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Cmd_to_LCD(IN IDX c, IN BYTE cmd);

/*--------------------------------------------------------------------------------------------*/
/*       (Display_ON).                                 */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Display_ON()            { Write_Cmd_to_LCD(0, 0x3f); Write_Cmd_to_LCD(1, 0x3f); }

/*--------------------------------------------------------------------------------------------*/
/*       (Display_OFF).                               */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Display_OFF()           { Write_Cmd_to_LCD(0, 0x3e); Write_Cmd_to_LCD(1, 0x3e); }

/*--------------------------------------------------------------------------------------------*/
/*           ,       */
/*     (Set_Start_Line).                                                   */
/* :    (0, 1),   (0..63).                                      */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Set_Start_Line(c, line) Write_Cmd_to_LCD(c, 0xc0 + (line))

/*--------------------------------------------------------------------------------------------*/
/*         (Page_Addr_Set).              */
/* :    (0, 1),   (0..7).                                     */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Page_Addr_Set(c, page)  Write_Cmd_to_LCD(c, 0xb8 + (page))

/*--------------------------------------------------------------------------------------------*/
/*         (Column_Addr_Set).             */
/* :    (0, 1),   (0..63).                                     */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Write_Cmd_Column_Addr_Set(c, col)    Write_Cmd_to_LCD(c, 0x40 + (col));

/*--------------------------------------------------------------------------------------------*/
/*   1     .                                          */
/* :    (0, 1),   .                                  */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Data_to_LCD(IN IDX c, IN BYTE data);

/*--------------------------------------------------------------------------------------------*/
/*   n     .                                         */
/* :    (0, 1),    ,   .    */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Data_to_LCD(IN IDX c, IN BYTE __flash *pData, IN BYTE n);
void Write_Data_to_LCD(IN IDX c, IN BYTE *pData, IN BYTE n);

/*--------------------------------------------------------------------------------------------*/
/*   n      .                                    */
/* :    (0, 1),  ,   .                      */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Data_to_LCD(IN IDX c, IN BYTE data, IN BYTE n);

/*--------------------------------------------------------------------------------------------*/
/*   n     .                                            */
/* :      (/),                                  */
/*                  ,   .             */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Write_Data_to_LCD(IN BYTE P, IN BYTE C, IN BYTE *pData, IN BYTE n);

/*--------------------------------------------------------------------------------------------*/
/*   .                                                                     */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void LCD_Clear(void);

/*--------------------------------------------------------------------------------------------*/
/*   1     .                                        */
/* :    (0, 1).                                                            */
/* :   .                                                               */
/*--------------------------------------------------------------------------------------------*/
OUT BYTE Read_Data_from_LCD(IN IDX c);

/*--------------------------------------------------------------------------------------------*/
/*   n     .                                       */
/* :    (0, 1),     ,  . .    */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Read_Data_from_LCD(IN IDX c, OUT BYTE *pData, IN BYTE n);

/*--------------------------------------------------------------------------------------------*/
/*    n  .                                               */
/* :      (/),                                  */
/*                  ,   .                 */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Read_Data_from_LCD(IN BYTE P, IN BYTE C, OUT BYTE *pData, IN BYTE n);

/*--------------------------------------------------------------------------------------------*/
/*       .                                  */
/* :      (),                                  */
/*                 ,                                                  */
/*                    flash,                                     */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   DRAW_DIRECT         0
#define   DRAW_XOR            (1 << 0)
#define   DRAW_INVERTED       (1 << 1)
#define   DRAW_UNDERLINED     (1 << 2)
void Draw_Bitmap(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H, IN BYTE __flash *pData, IN BYTE flags = DRAW_DIRECT);

/*--------------------------------------------------------------------------------------------*/
/*     .                                       */
/* :         ,                 */
/*                   ,                                 */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Fill_Rect(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H, IN BYTE flags = DRAW_DIRECT);

/*--------------------------------------------------------------------------------------------*/
/*      .                                            */
/* :      (),                                        */
/*                 .                                                  */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Put_Pixel(IN BYTE X, IN BYTE Y, IN BYTE flags = DRAW_DIRECT);

/*--------------------------------------------------------------------------------------------*/
/*      .                                      */
/* :        (),                               */
/*                 .                                                */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Draw_Line(IN BYTE X1, IN BYTE Y1, IN BYTE X2, IN BYTE Y2, IN BYTE flags = DRAW_DIRECT);

/*--------------------------------------------------------------------------------------------*/
/*   .                                                        */
/* :        ,                     */
/*                  .                                     */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
inline void Draw_Rect(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H)
{
  Draw_Line(X, Y, (X) + (W) - 1, Y); Draw_Line(X, (Y) + (H) - 1, (X) + (W) - 1, (Y) + (H) - 1);
  Draw_Line(X, Y, X, (Y) + (H) - 1); Draw_Line((X) + (W) - 1, Y, (X) + (W) - 1, (Y) + (H) - 1);
}

/*--------------------------------------------------------------------------------------------*/
/*     .                                        */
/* :         ,                     */
/*                   .                                     */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Clr_Rect(x, y, w, h)  Fill_Rect(x, y, w, h, DRAW_INVERTED)

/*--------------------------------------------------------------------------------------------*/
/*     .                                     */
/* :         ,                 */
/*                   .                                 */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Invert_Rect(IN BYTE X, IN BYTE Y, IN BYTE W, IN BYTE H);

/*--------------------------------------------------------------------------------------------*/
/*        .                      */
/* :    ,                                                                  */
/*                 (),                                      */
/*                   flash,                                     */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
#define   Draw_Char(ch, X, Y, font, flags)    \
  Draw_Bitmap(X, Y, (font)->Width, (font)->Height, Get_Char_Data_Addr(ch, font), flags)

/*--------------------------------------------------------------------------------------------*/
/*        .                          */
/* :    ,                                                                 */
/*                 (),                                      */
/*                   flash,                                     */
/*                 .                                            */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Draw_Text(IN char *str, IN BYTE X, IN BYTE Y, IN struct FONT __flash *font, IN BYTE flags = DRAW_DIRECT);
void Draw_Text(IN char __flash *str, IN BYTE X, IN BYTE Y, IN struct FONT __flash *font, IN BYTE flags = DRAW_DIRECT);

/*--------------------------------------------------------------------------------------------*/
/*      .                                           */
/* :     (   flash),                                               */
/*                   flash.                                     */
/* :     .                                                      */
/*--------------------------------------------------------------------------------------------*/
#define   Get_Text_Width(str, font)       Str_Len((str)) * (font)->Width

/*--------------------------------------------------------------------------------------------*/
/*      .                                          */
/* :     (   flash),                                               */
/*                   flash,                                     */
/* :     .                                                     */
/*--------------------------------------------------------------------------------------------*/
#define   Get_Text_Height(str, font)      (font)->Height

/*--------------------------------------------------------------------------------------------*/
/*      .                                         */
/* :        flash,                                     */
/* :    .                                                            */
/*--------------------------------------------------------------------------------------------*/
#define   Get_Char_Width(font)            (font)->Width

/*--------------------------------------------------------------------------------------------*/
/*      .                                         */
/* :        flash,                                     */
/* :    .                                                            */
/*--------------------------------------------------------------------------------------------*/
#define   Get_Char_Height(font)           (font)->Height

/*--------------------------------------------------------------------------------------------*/
/*          .    */
/* :    ,                                                                  */
/*                   flash,                                     */
/* :     .                                                */
/*--------------------------------------------------------------------------------------------*/
#define   Get_Char_Data_Addr(ch, font)    \
  (font)->pData + (ch) * (font)->Width * ((((font)->Height % 8) != 0) ? (1 + (font)->Height / 8) : ((font)->Height / 8))

/*--------------------------------------------------------------------------------------------*/
/*   .                                                                 */
/*--------------------------------------------------------------------------------------------*/
struct  FONT
{
  BYTE  Height;         //    .
  BYTE  Width;          //    .
  BYTE  Count;          //      .
  BYTE __flash *pData;  //      .
};

/*--------------------------------------------------------------------------------------------*/
/*  6  8 .                                                                      */
/*--------------------------------------------------------------------------------------------*/
extern __flash struct FONT Font_6x8;
extern __flash BYTE Font_6x8_Data[];

/*--------------------------------------------------------------------------------------------*/
/*  7  10  .                                                              */
/*--------------------------------------------------------------------------------------------*/
extern __flash struct FONT Font_7x10_thin;
extern __flash BYTE Font_7x10_thin_Data[];

/*--------------------------------------------------------------------------------------------*/
/*  7  10  .                                                          */
/*--------------------------------------------------------------------------------------------*/
extern __flash struct FONT Font_7x10_bold;
extern __flash BYTE Font_7x10_bold_Data[];

/*--------------------------------------------------------------------------------------------*/
/*  12  16 .                                                                    */
/*--------------------------------------------------------------------------------------------*/
extern __flash struct FONT Font_12x16;
extern __flash BYTE Font_12x16_Data[];


/**********************************************************************************************/
#endif
